//*****************************************************************************
//-----------------------------------------------------------------------------
// Fast simulation and reconstruction package for TOP counter (F77 core)
// M. Staric, Jan-2010
//-----------------------------------------------------------------------------
//
// topsim.cc
//     simulation: output to root file
//-----------------------------------------------------------------------------
//*****************************************************************************

#include <iostream>
#include <iomanip>
#include <math.h>
#include "TOPsimu.h"
#include "TOPreco.h"
#include "RandomParticle.h"
#include "TOPtrack.h"
#include "TOPhbook.h"
#include "TOProotX.h"

void TOPconfigure(int geo, int PMTchoice, int QEchoice, double Redge);
double Masses[3]={0.10566, 0.13957, 0.49368}; 
int HypID[3]={2,3,4};
double LogL[3], ExpPhot[3]; int Nphot;
int Nhyp=3;

static EventX event={0,0}; // event: num.of tracks, num.of photons
static TrackX track;       // track given at IP
static TrackX hit;         // track given at TOP counter
static DataX data;         // photon

using namespace std;

int main()
{
  int geo=2;
  int PMTchoice=1;
  int QEchoice=3;
  double sigT0=25e-3;
  double Bkg=0;
  char RootFile[80]="../root/top_reco.root";
  int ngen=1000;

  TOPconfigure(geo, PMTchoice, QEchoice, 0.);
  TOPsimu simu(sigT0, Bkg);
  RandomParticle p;
  p.setFrac(0, 1, 1, 1, 0);

 //create a Tree file
  TFile file(RootFile,"RECREATE");
 
  // Create ROOT Trees and branches
  TTree t1("t1","TOP simulation output: event");
  t1.Branch("event", &event,"ntra/I:ndata");

  TTree t2("t2","TOP simulation output: tracks");
  t2.Branch("track", &track,"x/F:y:z:px:py:pz:tlen:charge:lund/I");
  t2.Branch("hit", &hit,"x/F:y:z:px:py:pz:tlen:charge:lund/I:bar/I");

  TTree t3("t3","TOP simulation output: photons");
  t3.Branch("data", &data,"bar/I:ch:tdc:x/F:y:z:t");

  int QbarID, chID, TDC; double x, y, z, t;
  for(int iev=1; iev<=ngen; iev++){
    if(iev%1000==0) cout << "Event="<<iev<<endl;

    p.genNext(); // generate mu, pi or K from IP uniformly into 4pi, 0<p<5 GeV/c

    simu.Clear();
    simu.AddParticle(p);
    simu.Simulate();

    event.numTrack=0; event.numData=0; // reset counters 

    track.x = p.X();
    track.y = p.Y();
    track.z = p.Z();
    track.px = p.Px();
    track.py = p.Py();
    track.pz = p.Pz();
    track.tlen = 0;
    track.charge = p.Q();
    track.lund = p.Lund();
    TOPtrack trk(p);
    int barID = trk.toTop(); // propagate track to TOP counter
    hit.x = trk.X();
    hit.y = trk.Y();
    hit.z = trk.Z();
    hit.px = trk.Px();
    hit.py = trk.Py();
    hit.pz = trk.Pz();
    hit.tlen = trk.Tlen();
    hit.charge = trk.Q();
    hit.lund = trk.Lund();
    hit.QbarID = barID;
    
    if(barID>=0) { // fill trees
      t2.Fill(); event.numTrack++; // fill track and count
      for(int i=0; i<simu.DataSize(); i++){
	simu.GetData(i, QbarID, chID, TDC);
	simu.GetData(i, QbarID, x, y, z, t);
	data.QbarID=QbarID; 
	data.chID=chID; 
	data.TDC=TDC;
	data.x=x; 
	data.y=y; 
	data.z=z; 
	data.t=t;
	t3.Fill(); event.numData++; // fill photons and count them
      }
      t1.Fill(); // fill event (track and photon counters)
    }

  }

  file.Write();
  return 0;
}


