//*****************************************************************************
//-----------------------------------------------------------------------------
// Fast simulation and reconstruction package for TOP counter (F77 core)
// M. Staric, May-2009
//-----------------------------------------------------------------------------
//
// TOPhbook.h
// C++ interface to F77 functions: interface to hbook
// note: /PAWC/ is dimensioned in srcF77/hbook_open.F
//-----------------------------------------------------------------------------
//*****************************************************************************

#ifndef _TOPhbook_h
#define _TOPhbook_h

#include <cstring>
#include "TOPf77fun.h"
extern "C" {
  void hbook1_(int*,const char*,int*,float*,float*,float*,int );
  void hbook2_(int*,const char*,int*,float*,float*,int*,float*,float*,float*,int );
  void hbprof_(int*,const char*,int*,float*,float*,float*,float*,const char*,int ,int );
  void hfill_(int*,float*,float*,float*);
}


//----------------------------------
// open and close hbook output file:
//----------------------------------

inline void HbookOpen(const char *file)
{
  int len=strlen(file);
  hbook_open_(file, len);
}

inline void HbookClose()
{
  hbook_close_();
}

//----------------------------------
// On-flight booking ntuples:
//----------------------------------

inline void NtupleInit(int ID, const char *name)
{
  int len=strlen(name);
  ntup_init_(&ID, name, len);
} 

inline void NtupleFill(const char *varname, double Var)
{
  int len=strlen(varname);
  float var=Var;
  ntup_ffill_(varname, &var, len);
}

inline void NtupleFill(const char *varname, float var)
{
  int len=strlen(varname);
  ntup_ffill_(varname, &var, len);
}

inline void NtupleFill(const char *varname, int Var)
{
  int len=strlen(varname);
  float var=Var;
  ntup_ffill_(varname, &var, len);
}

inline void NtupleFlush()
{
  ntup_flush_();
}

//----------------------------------
// Histograms:
//----------------------------------

inline void hbook1(int ID, const char *name, int Nx, float xmi, float xma, float vmx=0)
{
  int len=strlen(name);
  hbook1_(&ID, name, &Nx, &xmi, &xma, &vmx, len);
}

inline void hbook2(int ID, const char *name, int Nx, float xmi, float xma, 
		   int Ny, float ymi, float yma, float vmx=0)
{
  int len=strlen(name);
  hbook2_(&ID, name, &Nx, &xmi, &xma, &Ny, &ymi, &yma, &vmx, len);
}

inline void hbprof(int ID, const char *name, int Nx, float xmi, float xma, 
		   float ymi, float yma, const char *opt=" ")
{
  int len=strlen(name);
  int lenopt=strlen(opt);
  hbprof_(&ID, name, &Nx, &xmi, &xma, &ymi, &yma, opt, len, lenopt);
}

inline void hfill(int ID, double X, double Y=0, double Wt=1)
{
  float x=X; float y=Y; float wt=Wt;
  hfill_(&ID, &x, &y, &wt);
}


#endif
