!-----------------------------------------------------------------------
! TOP counter for Belle: interface to TOP_PHOT
! M. Staric, mar-2009, sept-2011
! units: GeV, GeV/c, cm, ns, cm/ns, T, nm
!-----------------------------------------------------------------------

      INTEGER FUNCTION PHOT_GETNUM()

      IMPLICIT NONE
#include "TOP_PHOT.fi"
      
      PHOT_GETNUM=N_PHOT
      END


      SUBROUTINE PHOT_GET(K,MDN,ICH,X,Y,LR,T,E,THC,FIC,NX,NY)

      IMPLICIT NONE
      INTEGER K                 ! index (row#) in TOP_PHOT [in]
      INTEGER MDN               ! module ID number
      INTEGER ICH               ! channel ID number
      REAL X, Y                 ! position at exit window [cm]
      INTEGER LR                ! 0/1=left/right exit window
      REAL T, E                 ! TOP [ns], energy [eV]
      REAL THC, FIC             ! Cerenkov polar and azimuthal angles
      INTEGER NX, NY            ! (signed) number of reflections in x and y
#include "TOP_PHOT.fi"

      IF(K.GT.0.AND.K.LE.N_PHOT) THEN
         MDN=PHOT_M(K)
         ICH=PHOT_ICH(K)
         X=PHOT_R(1,K)
         Y=PHOT_R(2,K)
         LR=PHOT_LR(K)
         T=PHOT_T(K)
         E=PHOT_E(K)
         THC=PHOT_THC(K)
         FIC=PHOT_FIC(K)
         NX=PHOT_NX(K)
         NY=PHOT_NY(K)
      ELSE
         MDN=-1
         ICH=-1
         X=0
         Y=0
         LR=0
         T=0
         E=0
         THC=0
         FIC=0
         NX=0
         NY=0
      ENDIF
      END


      SUBROUTINE PHOT_GET_E(K,X,Y,Z,SX,SY,SZ,T,E)

      IMPLICIT NONE
      INTEGER K                 ! index (row#) in TOP_PHOT [in]
      REAL X, Y, Z              ! emission point [cm]
      REAL SX, SY, SZ           ! emission direction
      REAL T                    ! emission time (no T0jitter) [ns]
      REAL E                    ! energy [eV]
#include "TOP_PHOT.fi"

      IF(K.GT.0.AND.K.LE.N_PHOT) THEN
         X=PHOT_R0(1,K)
         Y=PHOT_R0(2,K)
         Z=PHOT_R0(3,K)
         SX=PHOT_DIR0(1,K)
         SY=PHOT_DIR0(2,K)
         SZ=PHOT_DIR0(3,K)
         T=PHOT_T0(K)
         E=PHOT_E(K)
      ELSE
         X=0
         Y=0
         Z=0
         SX=0
         SY=0
         SZ=0
         T=0
         E=0
      ENDIF

      END


      SUBROUTINE PHOT_GET_D(K,X,Y,Z,SX,SY,SZ,T,E)

      IMPLICIT NONE
      INTEGER K                 ! index (row#) in TOP_PHOT [in]
      REAL X, Y, Z              ! detection point [cm]
      REAL SX, SY, SZ           ! detection direction
      REAL T                    ! detection time (no TTS, no T0jitter) [ns]
      REAL E                    ! energy [eV]
#include "TOP_PHOT.fi"

      IF(K.GT.0.AND.K.LE.N_PHOT) THEN
         X=PHOT_R(1,K)
         Y=PHOT_R(2,K)
         Z=PHOT_R(3,K)
         SX=PHOT_DIR(1,K)
         SY=PHOT_DIR(2,K)
         SZ=PHOT_DIR(3,K)
         T=PHOT_T(K)
         E=PHOT_E(K)
      ELSE
         X=0
         Y=0
         Z=0
         SX=0
         SY=0
         SZ=0
         T=0
         E=0
      ENDIF

      END



